@@ -29,6 +29,7 @@
 
 #include <config.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdarg.h>
@@ -40,10 +41,9 @@
 #include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-fontmap.h>
-#include <libgnomeprint/gp-truetype-utils.h>
-#include <libgnomeprint/gp-tt-t1.h>
 #include <libgnomeprint/gnome-font-family.h>
 #include <libgnomeprint/gnome-print-encode.h>
+#include <libgnomeprint/ttsubset/gnome-print-tt-subset.h>
 
 enum {
 	PROP_0,
@@ -646,6 +646,55 @@ gnome_font_face_find_from_family_and_style (const guchar *family, const guchar *
 	return face;
 }
 
+/**
+ * gnome_font_face_find_from_filename:
+ * @filename: filename of a font face in the system font database
+ * @index_: index of the face within @filename. (Font formats such as
+ *          TTC/TrueType Collections can have multiple fonts within
+ *          a single file.
+ * 
+ * Looks up the #GnomeFontFace for a particular pair of filename and
+ * index of the font within the file. The font must already be within
+ * the system font database; this can't be used to access arbitrary
+ * fonts on disk.
+ * 
+ * Return value: the matching #GnomeFontFace, if any, otherwise %NULL
+ **/
+GnomeFontFace *
+gnome_font_face_find_from_filename (const guchar *filename, gint index_)
+{
+	GPFontMap * map;
+	GPFontEntry match_e;
+	GPFontEntry * e;
+
+	/* The hash table for map->filenamedict uses this GPFontEntry
+	 * itself as a key but only the file/index are used as input
+	 * to the hash and equal functions.
+	 */
+	match_e.file = (guchar *)filename;
+	match_e.index = index_;
+
+	map = gp_fontmap_get ();
+		
+	e = g_hash_table_lookup (map->filenamedict, &match_e);
+	if (!e) {
+		gp_fontmap_release (map);
+		return NULL;
+	}
+
+	if (e->face) {
+		gnome_font_face_ref (e->face);
+		gp_fontmap_release (map);
+		return e->face;
+	}
+
+	gff_face_from_entry (e);
+
+	gp_fontmap_release (map);
+
+	return (e->face);
+}
+
 GnomeFontFace *
 gnome_font_face_find_closest_from_pango_font (PangoFont *pfont)
 {
@@ -1340,184 +1389,55 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 {
 	GnomePrintBuffer b;
 	const gchar *file_name;
-	guchar *fbuf;
-	GSList *strings;
-	const gchar * embeddedname;
-	gdouble TTVersion, MfrRevision;
-	const ArtDRect *bbox;
-	gint i;
-	gchar *line;
-	gint line_length;
+	gint nglyphs, j, k, lower, upper, len;
+	guchar *subfont_file = NULL;
+	gushort glyphArray[256];
+        guchar encoding[256];
 
 	g_return_if_fail (pso->face->entry->type == GP_FONT_ENTRY_TRUETYPE);
  	file_name = pso->face->entry->file;
 
-	if (GNOME_PRINT_OK != gnome_print_buffer_mmap (&b, file_name)) {
+	nglyphs = pso->face->num_glyphs;
+
+	len = pso->encodedname ? strlen (pso->encodedname) : 0;
+	lower = (len > 3) ? atoi (pso->encodedname + len - 3) : 0;
+	upper = lower + 1;
+
+	k = 1;
+	lower *= 255;
+	upper *= 255;
+
+	glyphArray[0] = encoding[0] = 0;
+
+	for ( j = lower; j < upper && j < nglyphs; j++) {
+		if (PSO_GLYPH_MARKED (pso, j)) {
+			glyphArray [k] = j;
+			encoding [k] = j%255 + 1;
+			k++;
+		}
+	}
+
+	(void) gnome_print_ps_tt_create_subfont (file_name, pso->encodedname, &subfont_file, glyphArray, encoding, k);
+
+	if (GNOME_PRINT_OK != gnome_print_buffer_mmap (&b, subfont_file)) {
 		gnome_font_face_ps_embed_empty (pso);
-		return;
-	}
-	
-	embeddedname = pso->face->psname;
-
-	fbuf = b.buf;
-	strings = gp_tt_split_file (fbuf, b.buf_size, pso->face->entry->index);
-	if (strings) {
-		/* Relatively easy - just embed TTF into PS stream */
-		/* Download master font */
-		TTVersion = 1.0;
-		MfrRevision = 1.0;
-		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-");
-		gf_pso_print_double (pso, "%g", TTVersion);
-		gf_pso_sprintf (pso, "-");
-		gf_pso_print_double (pso, "%g", MfrRevision);
-		gf_pso_sprintf (pso, "\n");
-
-		gf_pso_sprintf (pso, "11 dict begin\n");
-		gf_pso_sprintf (pso, "/FontName (%s) cvn def\n", embeddedname);
-		gf_pso_sprintf (pso, "/Encoding 256 array\n");
-		gf_pso_sprintf (pso, "0 1 255 {1 index exch /.notdef put} for\n");
-		gf_pso_sprintf (pso, "readonly def\n");
-		gf_pso_sprintf (pso, "/PaintType 0 def\n");
-		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
-		/* fixme: */
-		bbox = gnome_font_face_get_stdbbox (pso->face);
-		gf_pso_sprintf (pso, "/FontBBox [");
-		gf_pso_print_double (pso, "%g",  bbox->x0);
-		gf_pso_sprintf (pso, " ");
-		gf_pso_print_double (pso, "%g", bbox->y0);
-		gf_pso_sprintf (pso, " ");
-		gf_pso_print_double (pso, "%g",  bbox->x1);
-		gf_pso_sprintf (pso, " ");
-		gf_pso_print_double (pso, "%g",  bbox->y1);
-		gf_pso_sprintf (pso, "] def\n");
-		gf_pso_sprintf (pso, "/FontType 42 def\n");
-		/* fixme: XUID */
-		/* fixme: Be more intelligent */
-		gf_pso_sprintf (pso, "/sfnts [\n");
-		line_length = gnome_print_encode_hex_wcs (TT_BLOCK_SIZE);
-		line = g_malloc (line_length);
-		while (strings) {
-			guint start, next;
-			guchar *s, *e;
-			start = GPOINTER_TO_UINT (strings->data);
-			strings = g_slist_remove (strings, strings->data);
-			next = strings ? GPOINTER_TO_UINT (strings->data) : b.buf_size;
-			gf_pso_sprintf (pso, "<\n");
-			s = b.buf + start;
-			e = b.buf + next;
-			while (s < e) {
-				gint block_size, len;
-				block_size = MIN (TT_BLOCK_SIZE, e - s);
-				len = gnome_print_encode_hex (s, line, block_size);
-				gf_pso_print_sized (pso, line, len);
-				s += block_size;
-			}
-			gf_pso_sprintf (pso, strings ? ">\n" : "00>\n");
-		}
-		g_free (line);
-		gf_pso_sprintf (pso, "] def\n");
-		/* fixme: Use CID or something */
-		gf_pso_sprintf (pso, "/CharStrings %d dict dup begin\n", pso->face->num_glyphs);
-		gf_pso_sprintf (pso, "/.notdef 0 def\n");
-		for (i = 1; i < pso->face->num_glyphs; i++) {
-			gf_pso_sprintf (pso, "/_%d %d def\n", i, i);
-		}
-		gf_pso_sprintf (pso, "end readonly def\n");
-		gf_pso_sprintf (pso, "FontName currentdict end definefont pop\n");
-	} else {
-		guchar *afm;
-		/* This is somewhat experimental - convert TTF to Type1 */
-		afm = ttf2pfa (pso->face->ft_face, embeddedname, pso->glyphs);
-		if (!afm) {
-			gnome_print_buffer_munmap (&b);
-			g_warning ("file %s: line %d: Cannot convert TTF %s to Type1", __FILE__, __LINE__, file_name);
-			gnome_font_face_ps_embed_empty (pso);
-			return;
-		}
-		/* We take over ownership of afm here (be careful) */
-		pso->buf = afm;
-		pso->bufsize = strlen (afm);
-		pso->length = pso->bufsize;
+		g_warning ("Could not parse TrueType font from %s\n", subfont_file);
+		goto ps_truetype_error;
 	}
 
-	gnome_print_buffer_munmap (&b);
+	if (b.buf_size < 8)
+		goto ps_truetype_error;
 
-	/*
-	 * We have font downloaded (hopefully)
-	 *
-	 * With CharStrings: .nodef _1 _2 _3
-	 *
-	 * Now we have to build usable 2-byte encoded font from it
-	 *
-	 */
+	gf_pso_print_sized (pso, b.buf, b.buf_size);
 
-	/* fixme: That is crap. We should use CID */
-	/* Bitch 'o' bitches (Lauris) ! */
-	if (pso->face->num_glyphs < 256) {
-		gint glyph;
-		/* 8-bit vector */
-		pso->encodedbytes = 1;
-		gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
-		gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
-		gf_pso_sprintf (pso, "/Encoding [\n");
-		for (glyph = 0; glyph < 256; glyph++) {
-			guint g;
-			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
-			if ((g == 0) || !PSO_GLYPH_MARKED (pso, glyph)) {
-				gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/.notdef\n" : "/.notdef ", g);
-			} else {
-				gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/_%d\n" : "/_%d ", g);
-			}
-		}
-		gf_pso_sprintf (pso, "] def currentdict end\n");
-		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
-	} else {
-		gint nfonts, nglyphs, i, j;
-		/* 16-bit vector */
-		pso->encodedbytes = 2;
-		nglyphs = pso->face->num_glyphs;
-		nfonts = (nglyphs + 255) >> 8;
+ps_truetype_error:
 
-		gf_pso_sprintf (pso, "32 dict begin\n");
-		/* Common entries */
-		gf_pso_sprintf (pso, "/FontType 0 def\n");
-		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
-		gf_pso_sprintf (pso, "/FontName (%s-Glyph-Composite) cvn def\n", embeddedname);
-		gf_pso_sprintf (pso, "/LanguageLevel 2 def\n");
-		/* Type 0 entries */
-		gf_pso_sprintf (pso, "/FMapType 2 def\n");
-		/* Bitch 'o' bitches */
-		gf_pso_sprintf (pso, "/FDepVector [\n");
-		for (i = 0; i < nfonts; i++) {
-			gf_pso_sprintf (pso, "(%s) cvn findfont dup length dict begin\n", embeddedname);
-			gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
-			gf_pso_sprintf (pso, "/Encoding [\n");
-			for (j = 0; j < 256; j++) {
-				gint glyph;
-				glyph = 256 * i + j;
-				if (glyph >= nglyphs)
-					glyph = 0;
-				if ((glyph == 0) || !PSO_GLYPH_MARKED (pso, glyph)) {
-					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/.notdef\n" : "/.notdef ");
-				} else {
-					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/_%d\n" : "/_%d ", glyph);
-				}
-			}
-			gf_pso_sprintf (pso, "] def\n");
-			gf_pso_sprintf (pso, "currentdict end (%s-Glyph-Page-%d) cvn exch definefont\n", embeddedname, i);
-		}
-		gf_pso_sprintf (pso, "] def\n");
-		gf_pso_sprintf (pso, "/Encoding [\n");
-		for (i = 0; i < 256; i++) {
-			gint fn;
-			fn = (i < nfonts) ? i : 0;
-			gf_pso_sprintf (pso, ((i & 0xf) == 0xf) ? "%d\n" : "%d  ", fn);
-		}
-		gf_pso_sprintf (pso, "] def\n");
-		gf_pso_sprintf (pso, "currentdict end\n");
-		gf_pso_sprintf (pso, "(%s) cvn exch definefont pop\n", pso->encodedname);
-	}
-}
+	if (b.buf)
+		gnome_print_buffer_munmap (&b);
+
+	if (subfont_file)
+		unlink (subfont_file);
+}	
 
 /**
  * gnome_font_face_ps_embed_empty:
