@@ -89,6 +89,8 @@ gp_fontmap_unref (GPFontMap * map)
 			g_hash_table_destroy (map->familydict);
 		if (map->fontdict)
 			g_hash_table_destroy (map->fontdict);
+		if (map->filenamedict)
+			g_hash_table_destroy (map->filenamedict);
 		if (map->familylist) {
 			g_hash_table_remove (familylist2map, map->familylist);
 			g_list_free (map->familylist);
@@ -188,7 +190,7 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	 *     strcmp (FT_MODULE_CLASS (face->driver)->module_name, "truetype") == 0;
 	 * which isn't prety either (Chema)
  	 */
-	c = rindex (fc_file, '.');
+	c = strrchr (fc_file, '.');
 	if (!c)
 		return NULL;
 	else if (strcasecmp (c, ".pfb") == 0)
@@ -210,11 +212,11 @@ fcpattern_to_gp_font_entry (FcPattern *font)
 	e->refcount     = 1;
 	e->face         = NULL;
 	e->speciesname  = g_strdup (fc_style);
-	e->weight       = g_strdup (fc_style);
 	e->familyname   = g_strdup (fc_family);
-	e->name         = g_strdup_printf ("%s %s", e->familyname, e->weight);
+	e->name         = g_strdup_printf ("%s %s", e->familyname, fc_style);
 	e->italic_angle = italic_angle;
 	e->is_alias     = FALSE;
+	e->unused       = NULL;
 	
 	result = FcPatternGetInteger (font, FC_WEIGHT, 0, &weight);
 	if (result == FcResultMatch)
@@ -335,6 +337,7 @@ gp_fontmap_load_fontconfig (GPFontMap *map)
 		e = fcpattern_to_gp_font_entry (font);
 		if (e) {
 			g_hash_table_insert (map->fontdict, e->name, e);
+			g_hash_table_insert (map->filenamedict, e, e);
 			map->num_fonts++;
 			map->fonts = g_slist_prepend (map->fonts, e);
 		}
@@ -383,6 +386,23 @@ gp_fontmap_sort (GPFontMap *map)
 	}
 }
 
+static guint
+filename_hash (gconstpointer v)
+{
+	const GPFontEntry *e = v;
+
+	return g_str_hash (e->file) ^ (e->index << 16);
+}
+
+static gboolean
+filename_equal (gconstpointer v1,
+		gconstpointer v2)
+{
+	const GPFontEntry *e1 = v1;
+	const GPFontEntry *e2 = v2;
+
+	return (e1->index == e2->index) && strcmp (e1->file, e2->file) == 0;
+}
 
 static GPFontMap *
 gp_fontmap_load (void)
@@ -393,6 +413,7 @@ gp_fontmap_load (void)
 	map->refcount  = 1;
 	map->num_fonts = 0;
 	map->fontdict   = g_hash_table_new (g_str_hash, g_str_equal);
+	map->filenamedict = g_hash_table_new (filename_hash, filename_equal);
 	map->familydict = g_hash_table_new (g_str_hash, g_str_equal);
 
 	gp_fontmap_load_fontconfig (map);
@@ -432,7 +453,6 @@ gp_font_entry_unref (GPFontEntry * entry)
 		my_g_free (entry->name);
 		my_g_free (entry->familyname);
 		my_g_free (entry->speciesname);
-		my_g_free (entry->weight);
 		my_g_free (entry->file);
 		g_free (entry);
 	}
