@@ -167,7 +167,7 @@ static gint gnome_print_pdf_set_dash (GnomePrintPdf *pdf);
 static gint gnome_print_pdf_set_color_real (GnomePrintPdf *pdf, PdfColorGroup color_group,
 					    gdouble r, gdouble g, gdouble b);
 
-static gint gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *font);
+static gint gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *font, gboolean subfont_flag, gint page);
 static void gnome_print_pdf_font_print_encoding (GnomePrintPdf *pdf, GnomePrintPdfFont *font);
 static void gnome_print_pdf_font_print_widths (GnomePrintPdf *pdf, GnomePrintPdfFont *font);
 static void gnome_print_pdf_font_print_lastchar (GnomePrintPdf *pdf, GnomePrintPdfFont *font);
@@ -593,12 +593,17 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 	for (s = 0; s < pgl->num_strings; s++) {
 		GnomePosString *ps;
 		GnomeFont *font;
-		gint i;
+		gint i, current_page;
 		gdouble Tm [6];
 
 		ps = pgl->strings + s;
 		font = gnome_rfont_get_font (ps->rfont);
-		gnome_print_pdf_set_font_real (pdf, font);
+
+		/* Currently only truetype subsetting supported */
+
+		if (!NEEDED_SUBSETTING(font)) {
+			gnome_print_pdf_set_font_real (pdf, font, FALSE, 0);
+		}
 		
 		ret = gnome_print_pdf_set_color_real (pdf,
 						      PDF_COLOR_GROUP_FILL,
@@ -623,16 +628,65 @@ gnome_print_pdf_glyphlist (GnomePrintContext *pc, const gdouble *ctm, GnomeGlyph
 		gnome_print_pdf_page_fprintf (pdf, " Tm" EOL);
 
 		/* Build string */
-		ret = gnome_print_pdf_page_fprintf (pdf, "(");
-		
-		for (i = ps->start; i < ps->start + ps->length; i++) {
-			gint glyph;
-			glyph = pgl->glyphs[i].glyph;
-			glyph = gnome_print_pdf_assign_code_to_glyph (pdf, glyph);
-			ret = gnome_print_pdf_page_fprintf (pdf, "\\%o", glyph);
-			g_return_val_if_fail (ret >= 0, ret);
-		}
-		
+		if (!NEEDED_SUBSETTING(font)) {
+			ret = gnome_print_pdf_page_fprintf (pdf, "(");
+			for (i = ps->start; i < ps->start + ps->length; i++) {
+				gint glyph;
+				glyph = pgl->glyphs[i].glyph;
+				glyph = gnome_print_pdf_assign_code_to_glyph (pdf, glyph);
+				ret = gnome_print_pdf_page_fprintf (pdf, "\\%o", glyph);
+				g_return_val_if_fail (ret >= 0, ret);
+			}
+		} else {
+			for (i = ps->start; i < ps->start + ps->length; i++) {
+
+				/* Glyph position in parent font */
+
+				gint master_glyph;
+
+				/* Glyph in subfont */
+
+				gint glyph;
+
+				/* Pages of 256 glyphs, including starting */
+				/* null glyph				   */ 
+
+				gint page;
+				
+				master_glyph = (gint)(pgl->glyphs[i].glyph);
+				page  = master_glyph / 255;
+
+				/* Initially */
+
+				if (i==ps->start) {
+					current_page = page;
+					gnome_print_pdf_set_font_real (pdf, font, TRUE, page);
+					ret = gnome_print_pdf_page_fprintf (pdf, "(");
+
+				/* There's a ttf subfont to be set*/
+
+				} else if (current_page != page) {
+					current_page = page;
+					(void)gnome_print_pdf_page_fprintf (pdf, ") Tj" EOL);
+					gnome_print_pdf_set_font_real (pdf, font, TRUE, page);
+					ret = gnome_print_pdf_page_fprintf (pdf, "(");
+				}
+
+				gnome_font_face_pso_mark_glyph (pdf->selected_font->pso, master_glyph);
+
+				/* For each sub ttf font the first glyph is  */
+				/* null and 255th one is a valid glyph, also */
+				/* 1 needed to be added to each subglyph     */
+				/* for taking into account first null glyph  */
+				/* added to every subfont	             */	
+
+				glyph = master_glyph ? master_glyph % 255 + 1 : master_glyph;
+
+				ret = gnome_print_pdf_page_fprintf (pdf, "\\%o", glyph);
+				g_return_val_if_fail (ret >= 0, ret);
+			}
+		}	
+
 		ret = gnome_print_pdf_page_fprintf (pdf, ") Tj" EOL);
 
 /* FIXME: We are not honoring xy movements of a glyphlist right now. But rather
@@ -1043,7 +1097,9 @@ gnome_print_pdf_write_fonts (GnomePrintPdf *pdf)
 	while (list) {
 		font = list->data;
 		gnome_print_pdf_font_print_widths   (pdf, font);
-		gnome_print_pdf_font_print_encoding (pdf, font);
+		/* '/Encoding' not needed when the font is symbolic */
+		if (!NEEDED_SUBSETTING(font))
+			gnome_print_pdf_font_print_encoding (pdf, font);
 		gnome_print_pdf_font_print_lastchar (pdf, font);
 		list = list->next;
 	}
@@ -1165,6 +1221,22 @@ gnome_print_pdf_close_write_last_objects (GnomePrintPdf *pdf)
 	return ret;
 }
 
+static void
+gnome_print_embed_all_pdf_fonts (GnomePrintPdf *pdf)
+{
+	g_return_if_fail (pdf != NULL);
+
+	GList *list;
+
+	list = pdf->fonts;
+	while (list) {
+		GnomePrintPdfFont *font;
+		font = list->data;
+		gnome_print_embed_pdf_font (pdf, font);
+		list = list->next;
+	}
+}
+
 
 static gint
 gnome_print_pdf_close (GnomePrintContext *pc)
@@ -1177,6 +1249,8 @@ gnome_print_pdf_close (GnomePrintContext *pc)
 
 	g_return_val_if_fail (pc->transport != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
+	gnome_print_embed_all_pdf_fonts (pdf);
+
 	page = pdf->pages ? pdf->pages->data : NULL;
 	if (!pdf->pages || !page || !page->shown) {
 		g_warning ("file %s: line %d: Closing PDF context without final showpage", __FILE__, __LINE__);
@@ -1276,7 +1350,12 @@ gnome_print_pdf_set_color_real (GnomePrintPdf *pdf,
 	if (fill) {
 		if (pdf->color_set_fill && (r == pdf->r) && (g == pdf->g) && (b == pdf->b))
 			goto gnome_print_pdf_set_color_stroke;
-		gnome_print_pdf_page_fprintf (pdf, "%.3g %.3g %.3g rg" EOL, r, g, b);
+		gnome_print_pdf_page_print_double (pdf, "%.3g", r);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%.3g", g);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%.3g", b);
+		gnome_print_pdf_page_fprintf (pdf, " rg" EOL);
 		pdf->r = r;
 		pdf->g = g;
 		pdf->b = b;
@@ -1287,7 +1366,12 @@ gnome_print_pdf_set_color_stroke:
 	if (stroke) {
 		if (pdf->color_set_stroke && (r == pdf->r_) && (g == pdf->g_) && (b == pdf->b_))
 			return GNOME_PRINT_OK;
-		gnome_print_pdf_page_fprintf (pdf, "%.3g %.3g %.3g RG" EOL, r, g, b);
+		gnome_print_pdf_page_print_double (pdf, "%.3g", r);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%.3g", g);
+		gnome_print_pdf_page_fprintf (pdf, " ");
+		gnome_print_pdf_page_print_double (pdf, "%.3g", b);
+		gnome_print_pdf_page_fprintf (pdf, " RG" EOL);
 		pdf->r_ = r;
 		pdf->g_ = g;
 		pdf->b_ = b;
@@ -1469,7 +1553,8 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 	if (font->is_type_1) {
 		embed_result = gnome_print_pdf_t1_embed (pdf, file_name, &fontfile_object_number);
 	} else {
-		embed_result = gnome_print_pdf_tt_embed (pdf, file_name, &fontfile_object_number);
+		/* We always subset tt fonts */
+		embed_result = gnome_print_pdf_tt_subset_embed (pdf, font, file_name, &fontfile_object_number);
 	}
 
 	if (embed_result != GNOME_PRINT_OK) {
@@ -1479,7 +1564,12 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 
 	ascent  = (gint) gnome_font_face_get_ascender (face);
 	descent = (gint) gnome_font_face_get_descender (face);
-	flags = gnome_font_face_get_pdf_flags (face);
+	if (NEEDED_SUBSETTING(font)) {
+		/* Symbol encoding for subsetted ttf fonts */
+		flags = 4;
+	} else { 
+		flags = gnome_font_face_get_pdf_flags (face);
+	}
 #ifdef GET_STEMS
 	gnome_print_pdf_type1_get_stems (face, &stemv, &stemh);
 #else
@@ -1522,7 +1612,7 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 				 "/ItalicAngle %d" EOL
 				 "/StemV %d" EOL
 				 "/XHeight %d" EOL,
-				 gnome_font_face_get_ps_name (font->face),
+				 font->pso->encodedname,
 				 italic_angle, stemv, xheight);
 
 
@@ -1537,70 +1627,11 @@ gnome_print_pdf_font_print_descriptor (GnomePrintPdf *pdf, GnomePrintPdfFont *fo
 	return GNOME_PRINT_OK;
 }
 		
-static gint
-gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
-{
-	GnomePrintPdfPage *page;
-	GnomePrintPdfFont *font = NULL;
-	GnomeFontFace *face;
-	GList *needle;
+void
+gnome_print_embed_pdf_font (GnomePrintPdf *pdf, GnomePrintPdfFont *font) {
 	gint object_number_descriptor = 0;
-	gint i;
 	gboolean plan_b = FALSE; /* TRUE if we could not embed the font */
 
-	face = gnome_font->face;
-	
-	/* Check if the font is aready in Memory  */
-	needle = pdf->fonts;
-	while (needle) {
-		font = needle->data;
-		if (font->face == face)
-			break;
-		needle = needle->next;
-	}
-	/* If we already created this font, just set the font */
-	if (needle != NULL)
-		goto gnome_print_pdf_set_font_real_page_stuff;
-
-	/* Create a new GnomePrintPdfFont */
-	font = g_new (GnomePrintPdfFont, 1);
-	font->face = gnome_font_get_face (gnome_font);
-	g_object_ref (font->face);
-	font->pso  = gnome_font_face_pso_new (font->face, NULL, 0);
-	font->is_basic_14 = FALSE;
-	font->nglyphs = gnome_font_face_get_num_glyphs (face);
-	font->glyph_to_code = g_hash_table_new (NULL, NULL);
-	font->code_to_glyph = g_new (gint, 0xff);
-	for (i = 0; i < 0xff; i++) {
-		font->code_to_glyph [i] = -1;
-	}
-	font->code_assigned = 0;
-	
-	/* Get object numbers for the font */
-	if (font->is_basic_14) {
-		font->object_number_encoding = -1;
-		font->object_number_widths   = -1;
-		font->object_number_lastchar = -1;
-	} else {
-		font->object_number_encoding = gnome_print_pdf_object_new (pdf);
-		font->object_number_widths   = gnome_print_pdf_object_new (pdf);
-		font->object_number_lastchar = gnome_print_pdf_object_new (pdf);
-	}
-	font->object_number = gnome_print_pdf_object_new (pdf);
-
-	/* Make sure this backend can handle this font type */
-	if (face->entry->type == GP_FONT_ENTRY_TYPE1)
-		font->is_type_1 = TRUE;
-	else if (face->entry->type == GP_FONT_ENTRY_TRUETYPE)
-		font->is_type_1 = FALSE;
-	else {
-		g_warning ("We only support True Type and Type 1 fonts for now");
-		return GNOME_PRINT_ERROR_UNKNOWN;
-	}
-
-	/* Add it to the list of fonts of this job */
-	pdf->fonts = g_list_prepend (pdf->fonts, font);
-
 	if (!font->is_basic_14) {
 		GnomePrintReturnCode retval;
 		retval =  gnome_print_pdf_font_print_descriptor (pdf, font,
@@ -1628,12 +1659,14 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 		gnome_print_pdf_fprintf  (pdf,
 					  "/FirstChar %d" EOL
 					  "/LastChar %d 0 R" EOL
-					  "/Encoding %i 0 R" EOL
 					  "/Widths %d 0 R" EOL,
 					  0,
 					  font->object_number_lastchar,
-					  font->object_number_encoding,
 					  font->object_number_widths);
+		if (!NEEDED_SUBSETTING(font))
+			gnome_print_pdf_fprintf  (pdf,
+					  "/Encoding %i 0 R" EOL,
+					  font->object_number_encoding);
 	}
 	if (!plan_b) {
 		gnome_print_pdf_fprintf (pdf,
@@ -1642,7 +1675,89 @@ gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font)
 	}
 	
 	gnome_print_pdf_object_end   (pdf, font->object_number, FALSE);
+}
 
+static gint
+gnome_print_pdf_set_font_real (GnomePrintPdf *pdf, const GnomeFont *gnome_font, gboolean subfont_flag, gint subfont_number)
+{
+	GnomePrintPdfPage *page;
+	GnomePrintPdfFont *font = NULL;
+	GnomeFontFace *face;
+	GList *needle;
+	guchar *encodedname;
+	gint i;
+
+	face = gnome_font->face;
+	
+	if (subfont_flag) {
+		if (subfont_number == 0)
+			encodedname = g_strdup_printf ("GnomeUni-%s", face->psname);
+		else
+			encodedname = g_strdup_printf ("GnomeUni-%s_%03d", face->psname, subfont_number);
+	}
+
+	/* Check if the font is aready in Memory  */
+	needle = pdf->fonts;
+	while (needle) {
+		font = needle->data;
+		if (subfont_flag) {
+			if  (!strcmp(font->pso->encodedname, encodedname))
+				break;
+		} else if (font->face == face)
+			break;
+			
+		needle = needle->next;
+	}
+	/* If we already created this font, just set the font */
+	g_free (encodedname);
+	if (needle != NULL)
+		goto gnome_print_pdf_set_font_real_page_stuff;
+
+	/* Create a new GnomePrintPdfFont */
+	font = g_new (GnomePrintPdfFont, 1);
+	font->face = gnome_font_get_face (gnome_font);
+	g_object_ref (font->face);
+	font->pso  = gnome_font_face_pso_new (font->face, NULL, subfont_number);
+	font->is_basic_14 = FALSE;
+	font->nglyphs = gnome_font_face_get_num_glyphs (face);
+	font->glyph_to_code = g_hash_table_new (NULL, NULL);
+
+	/* All 256 charaters can have glyphs
+	 * ie, 0-255, we need 256th postion to be -1
+	 */
+
+	font->code_to_glyph = g_new (gint, 257);
+	for (i = 0; i < 257; i++) {
+		font->code_to_glyph [i] = -1;
+	}
+	font->code_assigned = 0;
+	
+	/* Get object numbers for the font */
+	if (font->is_basic_14) {
+		font->object_number_encoding = -1;
+		font->object_number_widths   = -1;
+		font->object_number_lastchar = -1;
+	} else {
+		if (!subfont_flag)
+                	font->object_number_encoding = gnome_print_pdf_object_new (pdf);
+		font->object_number_widths   = gnome_print_pdf_object_new (pdf);
+		font->object_number_lastchar = gnome_print_pdf_object_new (pdf);
+	}
+	font->object_number = gnome_print_pdf_object_new (pdf);
+
+	/* Make sure this backend can handle this font type */
+	if (face->entry->type == GP_FONT_ENTRY_TYPE1)
+		font->is_type_1 = TRUE;
+	else if (face->entry->type == GP_FONT_ENTRY_TRUETYPE)
+		font->is_type_1 = FALSE;
+	else {
+		g_warning ("We only support True Type and Type 1 fonts for now");
+		return GNOME_PRINT_ERROR_UNKNOWN;
+	}
+
+
+	/* Add it to the list of fonts of this job */
+	pdf->fonts = g_list_prepend (pdf->fonts, font);
 gnome_print_pdf_set_font_real_page_stuff:
 	/* Is the font alraedy set? If it is just return */
 	if (pdf->selected_font == font)
@@ -1856,7 +1971,7 @@ gnome_print_pdf_page_write (GnomePrintPdf *pdf, gchar *text)
 	len = strlen (text);
 	page = pdf->pages->data;
 	if ((pdf->stream_used + len + 1) > pdf->stream_allocated) {
-		while ((pdf->stream_used + len) > pdf->stream_allocated) {
+		while ((pdf->stream_used + len + 1) > pdf->stream_allocated) {
 			pdf->stream_allocated += grow_size;
 			grow_size <<= 1;
 		}
