@@ -37,13 +37,18 @@
 #include <libgnomeprint/gnome-print-pdf.h>
 #include <libgnomeprint/gnome-print-frgba.h>
 
+#ifdef ENABLE_SVG
+#include <libgnomeprint/gnome-print-svg.h>
+#endif
+
+#ifdef HAVE_MMAP
 /* For the buffer stuff, remove when the buffer stuff is moved out here */
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-/* Endif */
+#endif /* HAVE_MMAP */
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 static void gnome_print_context_init (GnomePrintContext *pc);
@@ -501,6 +506,16 @@ gnome_print_context_new (GnomePrintConfig *config)
 		return pc;
 	}
 
+#ifdef ENABLE_SVG
+	if (strcmp (drivername, "gnome-print-svg") == 0) {
+		pc = gnome_print_svg_new (config);
+		if (pc == NULL)
+			return NULL;
+		g_free (drivername);
+		return pc;
+	}
+#endif
+
 	if (pc == NULL)
 		g_warning ("Could not create context for driver: %s", drivername);
 
@@ -513,16 +528,24 @@ gnome_print_context_new (GnomePrintConfig *config)
 void
 gnome_print_buffer_munmap (GnomePrintBuffer *b)
 {
+#ifndef HAVE_MMAP
+	g_warning ("mmap not supported");
+#else
 	if (b->buf)
 		munmap (b->buf, b->buf_size);
 	b->buf = NULL;
 	b->buf_size = 0;
+#endif
 }
 
 gint
 gnome_print_buffer_mmap (GnomePrintBuffer *b,
 			 const guchar *file_name)
 {
+#ifndef HAVE_MMAP
+	g_warning ("Can't mmap file %s", file_name);
+	return GNOME_PRINT_ERROR_UNKNOWN;
+#else
 	struct stat s;
 	gint fh;
 
@@ -549,4 +572,5 @@ gnome_print_buffer_mmap (GnomePrintBuffer *b,
 	}
 
 	return GNOME_PRINT_OK;
+#endif /* HAVE_MMAP */
 }
