@@ -29,12 +29,14 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <locale.h>
+#include <errno.h>
 #include "gpa-root.h"
 #include "gpa-utils.h"
 #include "gpa-config.h"
 #include "gpa-node-private.h"
 
+extern int errno;
+
 enum {MODIFIED, LAST_SIGNAL};
 static GObjectClass *parent_class;
 static guint node_signals [LAST_SIGNAL] = {0};
@@ -411,7 +413,7 @@ gpa_node_get_int_path_value (GPANode *node, const guchar *path, gint *value)
 gboolean
 gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *value)
 {
-	gchar *loc, *e;
+	gchar *e;
 	guchar *v;
 	
 	g_return_val_if_fail (node != NULL, FALSE);
@@ -425,22 +427,19 @@ gpa_node_get_length_path_value (GPANode *node, const guchar *path, gdouble *valu
 	if (v == NULL)
 		return FALSE;
 	
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-	*value = strtod (v, &e);
+	*value = g_ascii_strtod (v, &e);
+	/* Should we be skipping spaces between the digits and the unit? */
 	if (e) {
-		if (!strcmp (e, "mm")) {
+		if (!g_ascii_strncasecmp (e, "mm", 2)) {
 			*value *= MM2PT;
-		} else if (!strcmp (e, "cm")) {
+		} else if (!g_ascii_strncasecmp (e, "cm", 2)) {
 			*value *= CM2PT;
-		} else if (!strcmp (e, "in")) {
+		} else if (!g_ascii_strncasecmp (e, "in", 2)) {
 			*value *= IN2PT;
 		}
 	}
 
 	g_free (v);
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
 	
 	return TRUE;
 }
