@@ -32,6 +32,7 @@
 #include <dirent.h>
 
 #include <gmodule.h>
+#include <glibconfig.h>
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
 
@@ -231,7 +232,9 @@ gpa_printer_new_from_tree (xmlNodePtr tree)
 		goto gpa_printer_new_from_tree_error;
 	}
 
+#ifdef HAVE_LC_MESSAGES
 	lang = setlocale (LC_MESSAGES, NULL);
+#endif
 	node = tree->xmlChildrenNode;
 	for (; node != NULL; node = node->next) {
 
@@ -386,7 +389,7 @@ gpa_printer_list_load_from_module_dir (GPAList *printers, const gchar *dir_path)
 {
 	struct dirent *entry;
 	DIR *dir;
-	gint ext_len = strlen (LTDL_SHLIB_EXT);
+	gint ext_len = strlen (G_MODULE_SUFFIX);
 	g_assert (ext_len > 0);
 
 	if (!g_module_supported ()) {
@@ -406,11 +409,12 @@ gpa_printer_list_load_from_module_dir (GPAList *printers, const gchar *dir_path)
 		gint len;
 
 		len = strlen (entry->d_name);
-		
+
 		if (len < ext_len + 2) /* 2 = one char + 1 for '.'*/
 			continue;
 
-		if (strcmp (entry->d_name + len - ext_len, LTDL_SHLIB_EXT))
+		if (*(entry->d_name + len - ext_len - 1) != '.' || 
+		    strcmp (entry->d_name + len - ext_len, G_MODULE_SUFFIX))
 			continue;
 		
 		path = g_build_filename (dir_path, entry->d_name, NULL);
@@ -617,7 +621,8 @@ gpa_printer_new_stub (const gchar *id, const gchar *name,
 	printer = (GPAPrinter *) gpa_node_new (GPA_TYPE_PRINTER, id);
 	printer->name     = g_strdup (name);
 
-	printer->module_path = g_strdup (path);
+	if (path != NULL)
+		printer->module_path = g_strdup (path);
 	printer->is_complete = FALSE;
 
 	return (GPANode *) printer;	
@@ -680,42 +685,16 @@ gpa_printer_new (const gchar *id, const gchar *name, GPAModel *model, GPASetting
 	GPANode *check;
 	GPAList *list;
 
-	g_return_val_if_fail (id && id[0], NULL);
-	g_return_val_if_fail (name && name[0], NULL);
-	g_return_val_if_fail (model != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_MODEL (model), NULL);
-	g_return_val_if_fail (settings != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_SETTINGS (settings), NULL);
-	g_return_val_if_fail (gpa_initialized (), NULL);
-
-	check = gpa_printer_get_by_id (id);
-	if (check) {
-		g_warning ("Can't create printer \"%s\" because the id \"%s\" is already used", name, id);
-		gpa_node_unref (check);
+	printer = gpa_printer_new_stub (id, name, NULL);
+	if (printer == NULL)
+		return NULL;
+	
+	if (!gpa_printer_complete_stub (printer, model, settings)) {
+		gpa_node_unref ((GPANode *) printer);
+		printer = NULL;
 		return NULL;
 	}
 
-	list = gpa_list_new (GPA_TYPE_SETTINGS, "Settings", TRUE);
-
-	printer = (GPAPrinter *) gpa_node_new (GPA_TYPE_PRINTER, id);
-	printer->name     = g_strdup (name);
-	printer->model    = gpa_node_attach (GPA_NODE (printer),
-					     GPA_NODE (gpa_reference_new (GPA_NODE (model), "Model")));
-	printer->settings = gpa_node_attach (GPA_NODE (printer),
-					     GPA_NODE (list));
-	printer->is_complete = TRUE;
-
-	gpa_node_reverse_children (GPA_NODE (printer));
-
-	gpa_list_prepend     (list, GPA_NODE (settings));
-	gpa_list_set_default (list, GPA_NODE (settings));
-
-	settings->printer = gpa_reference_new (GPA_NODE (printer), "Printer");
-	/* We sink the model reference because we have a GPAReference to it
-	 * we take ownership of the settings, so we don't unref them
-	 */
-	gpa_node_unref (GPA_NODE (model));
-
 	if (!gpa_printer_verify ((GPANode *) printer)) {
 		g_warning ("The newly created printer %s could not be verified", id);
 		gpa_node_unref ((GPANode *) printer);
