@@ -34,6 +34,7 @@
 #include <libgnomeprint/gpa/gpa-printer.h>
 #include <libgnomeprint/gpa/gpa-option.h>
 #include <libgnomeprint/gpa/gpa-settings.h>
+#include <libgnomeprint/gpa/gpa-utils.h>
 
 #include <cups/cups.h>
 #include <cups/ppd.h>
@@ -462,16 +463,63 @@ gnome_print_cups_printer_list_append (gpointer printers_list,
 	
 	num_printers = cupsGetDests(&dests);
 	
-	for (i = 0; i < num_printers; i ++)
-		append_printer (GPA_LIST (printers_list),
-				dests[i].name, dests[i].is_default,
-				path);
-	
+	for (i = 0; i < num_printers; i ++) {
+		if (dests[i].instance == NULL)
+			append_printer (GPA_LIST (printers_list),
+					dests[i].name, 
+					dests[i].is_default,
+					path);
+	}
+
 	cupsFreeDests (num_printers, dests);
 	
 	return;
 }
 
+static void
+gnome_print_cups_adjust_settings (GPASettings * settings,
+				  int num_options,
+				  cups_option_t *options)
+{
+	int i;
+	const char *value;
+	
+	value = cupsGetOption ("PageSize", num_options, options);
+	if (value != NULL) {
+		gpa_node_set_path_value (GPA_NODE (settings),
+					 "Output.Media.PhysicalSize", 
+					 value);
+	}
+
+	value = cupsGetOption ("sides", num_options, options);
+	if (value != NULL) {
+		if (strcmp("two-sided-long-edge", value) == 0) {
+			gpa_node_set_path_value (GPA_NODE (settings),
+						 "Output.Job.Duplex", 
+						 "true");
+			gpa_node_set_path_value (GPA_NODE (settings),
+						 "Output.Job.Tumble", 
+						 "false");
+		}
+		if (strcmp("two-sided-short-edge", value) == 0) {
+			gpa_node_set_path_value (GPA_NODE (settings),
+						 "Output.Job.Duplex", 
+						 "true");
+			gpa_node_set_path_value (GPA_NODE (settings),
+						 "Output.Job.Tumble", 
+						 "true");
+		}
+		if (strcmp("one-sided", value) == 0) {
+			gpa_node_set_path_value (GPA_NODE (settings),
+						 "Output.Job.Duplex", 
+						 "false");
+		}
+	}
+/* 	for (i=0; i < num_options; i++) { */
+/* 		g_print ("Option %s = %s\n", options[i].name, options[i].value); */
+/* 	} */
+}
+
 
 /*  ------------- GPA load_data ------------- */
 
@@ -483,6 +531,9 @@ G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
 	const char *name   = printer->name;
 	ppd_file_t *ppd    = NULL;
 	gboolean success   = FALSE;
+	cups_dest_t *dests = NULL;
+	gint num_printers;
+	gint i;
 
 	if (printer->is_complete)
 		return;
@@ -504,9 +555,51 @@ G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
 	if (settings == NULL)
 		goto gpa_module_load_data_exit;
 
+	num_printers = cupsGetDests(&dests);
+	for (i = 0; i < num_printers; i ++) {
+		if ((dests[i].instance == NULL) && 
+		    (strcmp (dests[i].name, name) == 0)) {
+			gnome_print_cups_adjust_settings 
+				(GPA_SETTINGS (settings),
+				 dests[i].num_options,
+				 dests[i].options);
+			break;
+		}
+	}
+
 	success = gpa_printer_complete_stub (printer, model, 
 					     GPA_SETTINGS (settings));
 
+	/* Do we have to add any further instances */
+	{
+		for (i = 0; i < num_printers; i ++) {
+			if ((dests[i].instance != NULL) && 
+			    (strcmp (dests[i].name, name) == 0)) {
+				char *id = g_strdup_printf ("SetIdFromCups-%s",
+							    dests[i].instance);
+				settings = gpa_settings_new (model, 
+							     dests[i].instance,
+							     id);
+				g_free (id);
+				id = NULL;
+				if (settings != NULL) {
+					gnome_print_cups_adjust_settings 
+						(GPA_SETTINGS (settings),
+						 dests[i].num_options,
+						 dests[i].options);
+					gpa_list_prepend 
+						(GPA_NODE (printer->settings), 
+						 GPA_NODE (settings));
+					GPA_SETTINGS(settings)->printer 
+						= gpa_reference_new 
+						(GPA_NODE (printer), 
+						 "Printer");
+				}
+			}
+		}
+		
+	}
+
  gpa_module_load_data_exit:
 	if (success == FALSE) {
 		g_warning ("The data for the CUPS printer %s "
@@ -526,6 +619,7 @@ G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
 	if (filename)
 		g_free (filename);
 	
+/* 	gpa_utils_dump_tree (GPA_NODE (printer),5); */
 	return;
 }
 
